#!/usr/bin/python3

import sys
import string
import select
from socket import *

class Client():
    def __init__(self, address, csd):
        self.address = address
        self.sd = csd

def printHelp():
    print('Usage: {} <local port>'.format(sys.argv[0]))

def shutdown():
    sd.close()
    exit(0)

try:
    lport = int(sys.argv[1])
except:
    printHelp()
    exit(1)

sd = socket(AF_INET, SOCK_STREAM)
sd.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
sd.bind(('', lport))
sd.listen(16)

rfds = [sd, sys.stdin]

clients = {}

while True:
    csd = None
    msg = None

    try:
        rfd, _, _ = select.select(rfds, [], [])
    except:
        shutdown()

    if sys.stdin in rfd:
        # We have typed something to send
        msg = sys.stdin.readline()
        msg = 'Server >> {}'.format(msg)
    elif sd in rfd:
        # New incoming connection
        try:
            csd, address = sd.accept()
        except:
            shutdown()
        clients[csd] = Client(address, csd)
        rfds.append(csd)
        msg = 'new client connected from {}:{}\n'.format(address[0], address[1])
    else:
        c = clients[rfd[0]]
        csd = c.sd
        try:
            data = c.sd.recv(4096)
        except ConnectionResetError:
            pass
        
        if not data:
            # Client probably disconnected
            rfds.remove(c.sd)
            c.sd.close()
            msg = 'client at {}:{} disconnected\n'.format(c.address[0], c.address[1])
            clients.pop(c.sd)
        else:
            try:
                msg = data.decode()
            except:
                continue
            msg = ''.join(filter(lambda x: x in string.printable, msg))
            msg = '{}:{} >> {}'.format(address[0], address[1], msg)


    if msg is None:
        print('must not happen')
        continue

    if sys.stdin not in rfd:
        print(msg, end='')

    data = bytearray(msg, 'ascii')

    for socket, c in clients.items():
        if socket == csd:
            continue
        c.sd.send(data)


